function [Ek, Emk] = test_kriging(func, sampling_func, c)
% test_kriging(func, sample_func)
%                           - an example run of the kriging method
%
% func:                     - the function used for kriging
% sample_func:              - the function used for taking the sampling points
% c                         - the condition number

	warning off

	N = 2;
	lb = -2 * ones(N,1);
	ub = 2 * ones(N,1);
	Nkrig = N^4;
	tests = 100;

	angle = rand() * 360;
	A = [cos(angle) sin(angle); -sin(angle) cos(angle)] * [c 0; 0 1];
	func = @(X) rotated_function(func, A, X);

	% Select sample points
	Xs = sampling_func(Nkrig, N, lb, ub);
	for i=1:Nkrig
		Ys(i) = feval(func, Xs(i,:));
	end

	% Normalize points for Kriging, map to an interval between 0 and 100 (numerical reasons)
	Xmin = min(Xs);
	Xmax = max(Xs);
	Sk = 100 ./ (Xmax - Xmin);
	Xk = (Xs - repmat(Xmin, Nkrig, 1)) .* repmat(Sk, Nkrig, 1);
	Yk = (Ys' - min(Ys)) * (100 / (max(Ys) - min(Ys)));

	% Construct simple Kriging function f_krig
	[beta, s, theta, C, invC, lambda] = calibrate_kriging(Xk, Yk);
	f_krig = @(X) kriging_estimate(Xk, beta, s, theta, C, invC, lambda, (X - Xmin) .* Sk) / (100 / (max(Ys) - min(Ys))) + min(Ys);

	% Multiply by its transpose to obtain correlation matrix for Mahalanobis based Kriging (don't know why,.., yet)
	A = A * A';

	% Construct a Mahalanobis distance based Kriging function f_mahal 
	% that used the covariance matrix A
	[beta_mk, s_mk, theta_mk, C_mk, invC_mk, lambda_mk] = calibrate_mahal_kriging(Xk, Yk, A);
	f_mahal = @(X) mahal_kriging_estimate(Xk, beta_mk, s_mk, theta_mk, C_mk, invC_mk, lambda_mk, (X - Xmin) .* Sk, A) / (100 / (max(Ys) - min(Ys))) + min(Ys);

	% Takes a number of test points to compare prediction vs. actual function value
	for j=1:tests
		x = lb + (ub - lb) .* rand(N,1);
		F(j) = feval(func, x');
		Fk(j) = f_krig(x');
		Fmk(j) = f_mahal(x');
	end
	Ek = mean(abs(F - Fk));
	Emk = mean(abs(F - Fmk));

	% Plot approximated fitness vs. real-sampled fitness
	subplot(3,2,1)
	plot_func(func, N, lb, ub, 0.1 * ones(N,1));
	subplot(3,2,3)
	plot_func(f_krig, N, lb, ub, 0.1 * ones(N,1));
	hold on
	plot3(Xs(:,1), Xs(:,2), Ys, 'o')
	hold off
	subplot(3,2,5)
	plot_func(f_mahal, N, lb, ub, 0.1 * ones(N,1));
	hold on
	plot3(Xs(:,1), Xs(:,2), Ys, 'o')
	hold off
	subplot(3,2,4)
	plot(F, Fk, 'o')
	xlabel('F')
	ylabel('F_{kriging}')
	subplot(3,2,6)
	plot(F, Fmk, 'o')
	xlabel('F')
	ylabel('F_{mahalanobis}')
	drawnow()

	warning on
